/*
 * Copyright (c) 2014 Markus Friedl.  All rights reserved.
 * Copyright (c) 2018 Gleb Fotengauer-Malinovskiy.  All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "includes.h"

#include <sys/types.h>
#include <string.h>

#include "sshbuf.h"
#include "digest.h"
#include "ext-mac.h"

struct ssh_ext_mac_ctx {
	int			 alg;
	struct ssh_digest_ctx	*digest;
	struct ssh_digest_ctx	*octx;
};

size_t
ssh_ext_mac_bytes(int alg)
{
	return ssh_digest_bytes(alg);
}

struct ssh_ext_mac_ctx *
ssh_ext_mac_start(int alg)
{
	struct ssh_ext_mac_ctx	*ret;

	if ((ret = calloc(1, sizeof(*ret))) == NULL)
		return NULL;
	ret->alg = alg;
	if ((ret->digest = ssh_digest_start(alg)) == NULL ||
	    (ret->octx = ssh_digest_start(alg)) == NULL)
		goto fail;
	return ret;
fail:
	ssh_ext_mac_free(ret);
	return NULL;
}

int
ssh_ext_mac_init(struct ssh_ext_mac_ctx *ctx, const void *key, size_t klen)
{
	if (key != NULL && ssh_digest_set_key(ctx->octx, key, klen) < 0)
		return -1;

	/* (re)start with octx */
	if (ssh_digest_copy_state(ctx->octx, ctx->digest) < 0)
		return -1;

	return 0;
}

int
ssh_ext_mac_update(struct ssh_ext_mac_ctx *ctx, const void *m, size_t mlen)
{
	return ssh_digest_update(ctx->digest, m, mlen);
}

int
ssh_ext_mac_update_buffer(struct ssh_ext_mac_ctx *ctx, const struct sshbuf *b)
{
	return ssh_digest_update_buffer(ctx->digest, b);
}

int
ssh_ext_mac_final(struct ssh_ext_mac_ctx *ctx, u_char *d, size_t dlen)
{
	size_t len;

	len = ssh_digest_bytes(ctx->alg);
	if (dlen < len ||
	    ssh_digest_final(ctx->digest, d, dlen))
		return -1;

	return 0;
}

void
ssh_ext_mac_free(struct ssh_ext_mac_ctx *ctx)
{
	if (ctx != NULL) {
		ssh_digest_free(ctx->digest);
		ssh_digest_free(ctx->octx);
		explicit_bzero(ctx, sizeof(*ctx));
		free(ctx);
	}
}
